// Copyright 2022 schukai GmbH
// SPDX-License-Identifier: AGPL-3.0

package xflags

import (
	"strings"
)

func (c *cmd[C]) getCommandLevel() (*cmd[C], []string) {

	result := c

	path := []string{}
	path = append(path, c.name)

	for _, c := range c.commands {

		if c.flagSet.Parsed() {
			var p []string
			result, p = c.getCommandLevel()
			path = append(path, p...)
			break
		}
	}

	return result, path
}

func (s *Settings[C]) Help() string {
	return s.createHelp(s.command, []string{})
}

func (s *Settings[C]) createHelp(cmd *cmd[C], path []string) string {

	h := strings.Join(path, " ")
	if h != "" {
		h = " " + h + " "
	}
	var help string

	help = "Usage:" + h

	g := getFlagTable(cmd.settings.command.flagSet)
	if len(g) > 0 {
		help += "[global options] "
	}

	if len(cmd.commands) > 0 {
		help += "[command] "
	}

	help += "[arguments]"
	help += "\n"

	if len(g) > 0 {
		help += "\nGlobal Options:\n"
		help += strings.Join(g, "\n") + "\n"
	}

	if len(cmd.commands) > 0 {
		for _, c := range cmd.commands {
			help += "\nCommand: " + c.name + "\n"
			//help += fmt.Sprintf("  %s\t%s", c.name, c.tagMapping[tagDescription])
			s := getFlagTable(c.flagSet)

			if len(s) > 0 {
				help += "\nOptions:\n"
				help += strings.Join(s, "\n") + "\n"
			}

		}
	}
	return help
}

// Help returns the help text for the command
func (s *Settings[C]) ContextHelp() string {
	cmd, path := s.command.getCommandLevel()
	return s.createHelp(cmd, path)

}
